from flask import Flask, render_template, request, jsonify, send_from_directory
import os
import random
import uuid
import time
import logging

# Configurar logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

app = Flask(__name__)

# Configurar pasta de arquivos estáticos
@app.route('/static/<path:path>')
def send_static(path):
    return send_from_directory('static', path)

# Armazenar jogadores e jogos em memória (em produção, use um banco de dados)
players = {}
games = {}

# Função para gerar todas as peças de dominó
def generate_pieces():
    pieces = []
    for i in range(7):
        for j in range(i, 7):
            pieces.append([i, j])
    random.shuffle(pieces)
    return pieces

# Função para verificar se uma peça pode ser jogada
def can_play_piece(piece, board_ends):
    if not board_ends:
        return True
    return piece[0] in board_ends or piece[1] in board_ends

# Função para verificar se um jogador ganhou
def check_winner(game, player):
    if not game['player_pieces'][player]:
        return True
    return False

# Função para fazer jogada do bot
def bot_play(game):
    logger.info(f"Bot está jogando no jogo {game['id']}")
    
    bot_pieces = game['player_pieces']['Bot']
    board_ends = game['board_ends']
    
    # Resetar flags de jogada do bot
    game['bot_just_played'] = False
    game['bot_move'] = None
    
    # Se não houver peças, não pode jogar
    if not bot_pieces:
        logger.info("Bot não tem peças para jogar")
        return False
    
    # Se o tabuleiro estiver vazio, jogar qualquer peça
    if not game['board']:
        piece = bot_pieces[0]
        game['board'].append(piece)
        game['board_ends'] = [piece[0], piece[1]]
        bot_pieces.remove(piece)
        game['bot_just_played'] = True
        game['bot_move'] = piece
        logger.info(f"Bot jogou a primeira peça: {piece}")
        return True
    
    # Tentar encontrar uma peça jogável
    playable_pieces = []
    for piece in bot_pieces:
        if piece[0] == board_ends[0] or piece[0] == board_ends[1] or piece[1] == board_ends[0] or piece[1] == board_ends[1]:
            playable_pieces.append(piece)
    
    if playable_pieces:
        # Escolher uma peça aleatória entre as jogáveis
        piece = random.choice(playable_pieces)
        
        # Determinar onde jogar a peça
        if piece[0] == board_ends[0]:
            game['board'].insert(0, [piece[1], piece[0]])
            game['board_ends'][0] = piece[1]
            logger.info(f"Bot jogou {piece} à esquerda")
        elif piece[1] == board_ends[0]:
            game['board'].insert(0, [piece[0], piece[1]])
            game['board_ends'][0] = piece[0]
            logger.info(f"Bot jogou {piece} à esquerda")
        elif piece[0] == board_ends[1]:
            game['board'].append([piece[0], piece[1]])
            game['board_ends'][1] = piece[1]
            logger.info(f"Bot jogou {piece} à direita")
        elif piece[1] == board_ends[1]:
            game['board'].append([piece[1], piece[0]])
            game['board_ends'][1] = piece[0]
            logger.info(f"Bot jogou {piece} à direita")
        
        bot_pieces.remove(piece)
        game['bot_just_played'] = True
        game['bot_move'] = piece
        return True
    
    # Se não puder jogar, comprar peça se houver disponível
    if game['pieces']:
        new_piece = game['pieces'].pop()
        bot_pieces.append(new_piece)
        logger.info(f"Bot comprou uma peça: {new_piece}")
        
        # Verificar se a nova peça pode ser jogada
        if new_piece[0] == board_ends[0] or new_piece[0] == board_ends[1] or new_piece[1] == board_ends[0] or new_piece[1] == board_ends[1]:
            # Jogar a peça imediatamente
            if new_piece[0] == board_ends[0]:
                game['board'].insert(0, [new_piece[1], new_piece[0]])
                game['board_ends'][0] = new_piece[1]
            elif new_piece[1] == board_ends[0]:
                game['board'].insert(0, [new_piece[0], new_piece[1]])
                game['board_ends'][0] = new_piece[0]
            elif new_piece[0] == board_ends[1]:
                game['board'].append([new_piece[0], new_piece[1]])
                game['board_ends'][1] = new_piece[1]
            elif new_piece[1] == board_ends[1]:
                game['board'].append([new_piece[1], new_piece[0]])
                game['board_ends'][1] = new_piece[0]
            
            bot_pieces.remove(new_piece)
            game['bot_just_played'] = True
            game['bot_move'] = new_piece
            logger.info(f"Bot jogou a peça que comprou: {new_piece}")
            return True
    
    # Se não puder jogar nem comprar, passar a vez
    logger.info("Bot não pode jogar e passou a vez")
    return False

# Rotas
@app.route('/')
def index():
    return render_template('index.html')

@app.route('/register', methods=['POST'])
def register():
    data = request.json
    username = data.get('username')
    
    if not username:
        return jsonify({'success': False, 'message': 'Nome de usuário é obrigatório'})
    
    # Verificar se o usuário já existe
    for player_id, player in players.items():
        if player['username'] == username:
            return jsonify({
                'success': True, 
                'player_id': player_id,
                'username': player['username'],
                'balance': player['balance']
            })
    
    # Criar novo usuário
    player_id = str(uuid.uuid4())
    players[player_id] = {
        'username': username,
        'balance': 1000  # Saldo inicial
    }
    
    return jsonify({
        'success': True, 
        'player_id': player_id,
        'username': username,
        'balance': 1000
    })

@app.route('/create_game', methods=['POST'])
def create_game():
    data = request.json
    game_type = data.get('game_type', '1v1')
    bet_amount = float(data.get('bet_amount', 10))
    
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    # Verificar se o usuário tem saldo suficiente
    player = None
    for p in players.values():
        if p['username'] == username:
            player = p
            break
    
    if not player or player['balance'] < bet_amount:
        return jsonify({'success': False, 'message': 'Saldo insuficiente'})
    
    # Deduzir o valor da aposta
    player['balance'] -= bet_amount
    
    game_id = str(uuid.uuid4())[:8]  # ID curto para facilitar
    
    games[game_id] = {
        'id': game_id,
        'type': game_type,
        'bet_amount': bet_amount,
        'status': 'waiting',  # waiting, playing, finished
        'players': [{'username': username, 'is_bot': False}],
        'current_player': None,
        'winner': None,
        'board': [],
        'board_ends': [],
        'pieces': generate_pieces(),
        'player_pieces': {username: []},
        'created_at': time.time()
    }
    
    # Distribuir peças para o jogador
    player_pieces = []
    for _ in range(7):
        if games[game_id]['pieces']:
            piece = games[game_id]['pieces'].pop()
            player_pieces.append(piece)
    
    games[game_id]['player_pieces'][username] = player_pieces
    
    return jsonify({'success': True, 'game_id': game_id})

@app.route('/create_bot_game', methods=['POST'])
def create_bot_game():
    data = request.json
    bet_amount = float(data.get('bet_amount', 10))
    difficulty = data.get('difficulty', 'normal')
    
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    # Verificar se o usuário tem saldo suficiente
    player = None
    for p in players.values():
        if p['username'] == username:
            player = p
            break
    
    if not player or player['balance'] < bet_amount:
        return jsonify({'success': False, 'message': 'Saldo insuficiente'})
    
    # Deduzir o valor da aposta
    player['balance'] -= bet_amount
    
    game_id = str(uuid.uuid4())[:8]
    
    # Criar jogo com bot - GARANTIR QUE O JOGADOR HUMANO COMECE
    games[game_id] = {
        'id': game_id,
        'type': 'bot',
        'bet_amount': bet_amount,
        'status': 'playing',  # Começar jogando imediatamente
        'players': [
            {'username': username, 'is_bot': False},
            {'username': 'Bot', 'is_bot': True}
        ],
        'current_player': username,  # JOGADOR HUMANO SEMPRE COMEÇA
        'winner': None,
        'board': [],
        'board_ends': [],
        'pieces': generate_pieces(),
        'player_pieces': {username: [], 'Bot': []},
        'bot_difficulty': difficulty,
        'bot_just_played': False,
        'bot_move': None,
        'created_at': time.time()
    }
    
    # Distribuir peças para o jogador e o bot
    player_pieces = []
    bot_pieces = []
    
    for _ in range(7):
        if games[game_id]['pieces']:
            piece = games[game_id]['pieces'].pop()
            player_pieces.append(piece)
    
    for _ in range(7):
        if games[game_id]['pieces']:
            piece = games[game_id]['pieces'].pop()
            bot_pieces.append(piece)
    
    games[game_id]['player_pieces'][username] = player_pieces
    games[game_id]['player_pieces']['Bot'] = bot_pieces
    
    logger.info(f"Jogo contra bot criado: {game_id}, jogador: {username}")
    
    return jsonify({'success': True, 'game_id': game_id})

@app.route('/join_game', methods=['POST'])
def join_game():
    data = request.json
    game_id = data.get('game_id')
    
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    if not game_id or game_id not in games:
        return jsonify({'success': False, 'message': 'Jogo não encontrado'})
    
    game = games[game_id]
    
    # Verificar se o jogo já está cheio
    if len(game['players']) >= 2:
        return jsonify({'success': False, 'message': 'Jogo já está cheio'})
    
    # Verificar se o jogador já está no jogo
    if any(p['username'] == username for p in game['players']):
        return jsonify({'success': False, 'message': 'Você já está neste jogo'})
    
    # Verificar se o usuário tem saldo suficiente
    player = None
    for p in players.values():
        if p['username'] == username:
            player = p
            break
    
    if not player or player['balance'] < game['bet_amount']:
        return jsonify({'success': False, 'message': 'Saldo insuficiente'})
    
    # Deduzir o valor da aposta
    player['balance'] -= game['bet_amount']
    
    # Adicionar jogador ao jogo
    game['players'].append({'username': username, 'is_bot': False})
    
    # Distribuir peças para o jogador
    player_pieces = []
    for _ in range(7):
        if game['pieces']:
            piece = game['pieces'].pop()
            player_pieces.append(piece)
    
    game['player_pieces'][username] = player_pieces
    
    # Iniciar o jogo se estiver cheio
    if len(game['players']) == 2:
        game['status'] = 'playing'
        game['current_player'] = game['players'][0]['username']
        logger.info(f"Jogo {game_id} iniciado com jogadores {[p['username'] for p in game['players']]}")
    
    return jsonify({'success': True})

@app.route('/game_state', methods=['GET'])
def game_state():
    game_id = request.args.get('game_id')
    
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    if not game_id or game_id not in games:
        return jsonify({'success': False, 'message': 'Jogo não encontrado'})
    
    game = games[game_id]
    
    # GARANTIR que o jogador humano comece em jogos contra bot
    if game['type'] == 'bot' and not game['board'] and game['current_player'] == 'Bot':
        game['current_player'] = username
        logger.info(f"Corrigindo jogo {game_id}: jogador {username} deve começar")
    
    # Construir estado do jogo para o jogador
    player_pieces_count = {}
    for player_name, pieces in game['player_pieces'].items():
        player_pieces_count[player_name] = len(pieces)
    
    players_info = []
    for player in game['players']:
        player_name = player['username']
        players_info.append({
            'username': player_name,
            'is_bot': player.get('is_bot', False),
            'pieces_count': player_pieces_count.get(player_name, 0)
        })
    
    # Verificar se é a vez do jogador
    is_your_turn = game['current_player'] == username
    
    # CORREÇÃO: Garantir que o status e o jogador atual estejam corretos
    if game['type'] == 'bot' and len(game['players']) == 2:
        # Garantir que o jogo esteja em andamento
        if game['status'] == 'waiting':
            game['status'] = 'playing'
            logger.info(f"Corrigindo jogo {game_id}: status deve ser 'playing'")
        
        # Garantir que haja um jogador atual definido
        if not game['current_player']:
            game['current_player'] = username
            logger.info(f"Corrigindo jogo {game_id}: definindo jogador atual como {username}")
    
    return jsonify({
        'id': game['id'],
        'type': game['type'],
        'bet_amount': game['bet_amount'],
        'status': game['status'],
        'players': players_info,
        'current_player': game['current_player'],
        'winner': game['winner'],
        'board': game['board'],
        'board_ends': game['board_ends'],
        'your_hand': game['player_pieces'].get(username, []),
        'is_your_turn': is_your_turn,
        'bot_just_played': game.get('bot_just_played', False),
        'bot_move': game.get('bot_move', None)
    })

@app.route('/make_move', methods=['POST'])
def make_move():
    data = request.json
    game_id = data.get('game_id')
    piece = data.get('piece')
    position = data.get('position')
    
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    if not game_id or game_id not in games:
        return jsonify({'success': False, 'message': 'Jogo não encontrado'})
    
    game = games[game_id]
    
    # Verificar se é a vez do jogador
    if game['current_player'] != username:
        return jsonify({'success': False, 'message': 'Não é sua vez de jogar'})
    
    # Verificar se o jogador tem a peça
    if piece not in game['player_pieces'][username]:
        return jsonify({'success': False, 'message': 'Você não tem essa peça'})
    
    # Verificar se a peça pode ser jogada
    if game['board']:
        if position == 'left':
            if piece[0] != game['board_ends'][0] and piece[1] != game['board_ends'][0]:
                return jsonify({'success': False, 'message': 'Jogada inválida'})
        elif position == 'right':
            if piece[0] != game['board_ends'][1] and piece[1] != game['board_ends'][1]:
                return jsonify({'success': False, 'message': 'Jogada inválida'})
    
    # Jogar a peça
    game['player_pieces'][username].remove(piece)
    
    if not game['board']:
        # Primeira peça no tabuleiro
        game['board'].append(piece)
        game['board_ends'] = [piece[0], piece[1]]
    else:
        if position == 'left':
            if piece[0] == game['board_ends'][0]:
                game['board'].insert(0, [piece[1], piece[0]])
                game['board_ends'][0] = piece[1]
            else:
                game['board'].insert(0, [piece[0], piece[1]])
                game['board_ends'][0] = piece[0]
        else:  # right
            if piece[0] == game['board_ends'][1]:
                game['board'].append([piece[0], piece[1]])
                game['board_ends'][1] = piece[1]
            else:
                game['board'].append([piece[1], piece[0]])
                game['board_ends'][1] = piece[0]
    
    # Verificar se o jogador ganhou
    if check_winner(game, username):
        game['status'] = 'finished'
        game['winner'] = username
        
        # Atualizar saldo do jogador
        for player in players.values():
            if player['username'] == username:
                player['balance'] += game['bet_amount'] * 2
                break
        
        return jsonify({'success': True})
    
    # Passar a vez para o próximo jogador
    current_index = next((i for i, p in enumerate(game['players']) if p['username'] == username), 0)
    next_index = (current_index + 1) % len(game['players'])
    game['current_player'] = game['players'][next_index]['username']
    
    # Se o próximo jogador for um bot, fazer a jogada do bot automaticamente
    if game['type'] == 'bot' and game['current_player'] == 'Bot':
        # Pequeno atraso para simular "pensamento" do bot
        time.sleep(0.5)
        
        # Fazer a jogada do bot
        bot_played = bot_play(game)
        logger.info(f"Bot jogou automaticamente: {bot_played}")
        
        # Verificar se o bot ganhou
        if check_winner(game, 'Bot'):
            game['status'] = 'finished'
            game['winner'] = 'Bot'
            logger.info(f"Bot ganhou o jogo {game_id}")
        else:
            # Passar a vez de volta para o jogador humano
            game['current_player'] = username
    
    return jsonify({'success': True})

@app.route('/draw_piece', methods=['POST'])
def draw_piece():
    data = request.json
    game_id = data.get('game_id')
    
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    if not game_id or game_id not in games:
        return jsonify({'success': False, 'message': 'Jogo não encontrado'})
    
    game = games[game_id]
    
    # Verificar se é a vez do jogador
    if game['current_player'] != username:
        return jsonify({'success': False, 'message': 'Não é sua vez de jogar'})
    
    # Verificar se há peças disponíveis
    if not game['pieces']:
        # Passar a vez para o próximo jogador
        current_index = next((i for i, p in enumerate(game['players']) if p['username'] == username), 0)
        next_index = (current_index + 1) % len(game['players'])
        game['current_player'] = game['players'][next_index]['username']
        
        # Se o próximo jogador for um bot, fazer a jogada do bot
        if game['type'] == 'bot' and game['current_player'] == 'Bot':
            bot_played = bot_play(game)
            
            # Verificar se o bot ganhou
            if check_winner(game, 'Bot'):
                game['status'] = 'finished'
                game['winner'] = 'Bot'
            else:
                # Passar a vez de volta para o jogador humano
                game['current_player'] = username
        
        return jsonify({'success': True, 'message': 'Não há mais peças disponíveis'})
    
    # Comprar uma peça
    new_piece = game['pieces'].pop()
    game['player_pieces'][username].append(new_piece)
    
    # Verificar se o jogador pode jogar com a nova peça
    can_play = can_play_piece(new_piece, game['board_ends'])
    
    # Se não puder jogar, passar a vez
    if not can_play:
        # Passar a vez para o próximo jogador
        current_index = next((i for i, p in enumerate(game['players']) if p['username'] == username), 0)
        next_index = (current_index + 1) % len(game['players'])
        game['current_player'] = game['players'][next_index]['username']
        
        # Se o próximo jogador for um bot, fazer a jogada do bot
        if game['type'] == 'bot' and game['current_player'] == 'Bot':
            bot_played = bot_play(game)
            
            # Verificar se o bot ganhou
            if check_winner(game, 'Bot'):
                game['status'] = 'finished'
                game['winner'] = 'Bot'
            else:
                # Passar a vez de volta para o jogador humano
                game['current_player'] = username
    
    return jsonify({'success': True})

@app.route('/reload_balance', methods=['POST'])
def reload_balance():
    data = request.json
    username = data.get('username')
    amount = data.get('amount', 1000)
    
    if not username:
        return jsonify({'success': False, 'message': 'Nome de usuário é obrigatório'})
    
    # Encontrar o jogador e recarregar o saldo
    for player in players.values():
        if player['username'] == username:
            player['balance'] += amount
            return jsonify({
                'success': True,
                'username': username,
                'balance': player['balance'],
                'message': f'Saldo recarregado com {amount} créditos'
            })
    
    return jsonify({'success': False, 'message': 'Usuário não encontrado'})

@app.route('/force_bot_play', methods=['POST'])
def force_bot_play():
    data = request.json
    game_id = data.get('game_id')
    
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    if not game_id or game_id not in games:
        return jsonify({'success': False, 'message': 'Jogo não encontrado'})
    
    game = games[game_id]
    
    # Verificar se é um jogo contra bot
    if game['type'] != 'bot':
        return jsonify({'success': False, 'message': 'Este não é um jogo contra bot'})
    
    # Forçar a vez do bot se necessário
    if game['current_player'] != 'Bot':
        logger.info(f"Forçando a vez do bot no jogo {game_id}")
        game['current_player'] = 'Bot'
    
    # Fazer a jogada do bot
    bot_played = bot_play(game)
    logger.info(f"Bot jogou: {bot_played}")
    
    # Verificar se o bot ganhou
    if check_winner(game, 'Bot'):
        game['status'] = 'finished'
        game['winner'] = 'Bot'
        logger.info(f"Bot ganhou o jogo {game_id}")
    else:
        # Passar a vez de volta para o jogador humano
        game['current_player'] = username
    
    return jsonify({'success': True, 'bot_played': bot_played})

@app.route('/reset_game_state', methods=['POST'])
def reset_game_state():
    data = request.json
    game_id = data.get('game_id')
    
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    if not game_id or game_id not in games:
        return jsonify({'success': False, 'message': 'Jogo não encontrado'})
    
    game = games[game_id]
    
    # Verificar se é um jogo contra bot
    if game['type'] != 'bot':
        return jsonify({'success': False, 'message': 'Apenas jogos contra bot podem ser reiniciados'})
    
    # Forçar o estado do jogo para "playing" e definir o jogador atual
    game['status'] = 'playing'
    game['current_player'] = username
    
    return jsonify({'success': True})

@app.route('/start_bot_game', methods=['POST'])
def start_bot_game():
    data = request.json
    game_id = data.get('game_id')
    
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    if not game_id or game_id not in games:
        return jsonify({'success': False, 'message': 'Jogo não encontrado'})
    
    game = games[game_id]
    
    # Verificar se é um jogo contra bot
    if game['type'] != 'bot':
        return jsonify({'success': False, 'message': 'Este não é um jogo contra bot'})
    
    # Verificar se o jogo já começou
    if game['status'] == 'playing' and game['board']:
        return jsonify({'success': False, 'message': 'O jogo já começou'})
    
    # Definir o status como 'playing' e o jogador atual
    game['status'] = 'playing'
    
    # Jogador humano sempre começa
    game['current_player'] = username
    logger.info(f"Jogo contra bot {game_id} iniciado, jogador {username} começa")
    
    return jsonify({'success': True, 'first_player': username})

@app.route('/available_games', methods=['GET'])
def available_games():
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    # Filtrar jogos disponíveis (status 'waiting' e não criados pelo usuário atual)
    available = []
    for game_id, game in games.items():
        if (game['status'] == 'waiting' and 
            len(game['players']) < 2 and 
            not any(p['username'] == username for p in game['players'])):
            available.append({
                'id': game_id,
                'creator': game['players'][0]['username'],
                'bet_amount': game['bet_amount'],
                'created_at': game['created_at']
            })
    
    return jsonify({'success': True, 'games': available})

@app.route('/my_games', methods=['GET'])
def my_games():
    # Obter o nome de usuário do cabeçalho
    username = request.headers.get('X-Player-Username')
    if not username:
        return jsonify({'success': False, 'message': 'Usuário não identificado'})
    
    # Filtrar jogos do usuário
    my_games_list = []
    for game_id, game in games.items():
        if any(p['username'] == username for p in game['players']):
            my_games_list.append({
                'id': game_id,
                'status': game['status'],
                'players': [p['username'] for p in game['players']],
                'bet_amount': game['bet_amount'],
                'created_at': game['created_at']
            })
    
    return jsonify({'success': True, 'games': my_games_list})

@app.route('/test_bot_game', methods=['GET'])
def test_bot_game():
    """
    Rota para testar o jogo contra bot.
    Simula um jogo completo e retorna os resultados.
    """
    # Criar um usuário de teste
    test_username = f"test_user_{int(time.time())}"
    player_id = str(uuid.uuid4())
    players[player_id] = {
        'username': test_username,
        'balance': 1000
    }
    
    # Criar um jogo contra bot
    game_id = str(uuid.uuid4())[:8]
    bet_amount = 10
    
    # Configurar o jogo
    games[game_id] = {
        'id': game_id,
        'type': 'bot',
        'bet_amount': bet_amount,
        'status': 'playing',
        'players': [
            {'username': test_username, 'is_bot': False},
            {'username': 'Bot', 'is_bot': True}
        ],
        'current_player': test_username,
        'winner': None,
        'board': [],
        'board_ends': [],
        'pieces': generate_pieces(),
        'player_pieces': {test_username: [], 'Bot': []},
        'bot_difficulty': 'normal',
        'bot_just_played': False,
        'bot_move': None,
        'created_at': time.time(),
        'test_log': []  # Para registrar as ações do jogo
    }
    
    # Distribuir peças
    for _ in range(7):
        if games[game_id]['pieces']:
            piece = games[game_id]['pieces'].pop()
            games[game_id]['player_pieces'][test_username].append(piece)
    
    for _ in range(7):
        if games[game_id]['pieces']:
            piece = games[game_id]['pieces'].pop()
            games[game_id]['player_pieces']['Bot'].append(piece)
    
    # Registrar as peças iniciais
    game = games[game_id]
    game['test_log'].append({
        'action': 'game_created',
        'player_pieces': list(game['player_pieces'][test_username]),
        'bot_pieces': list(game['player_pieces']['Bot']),
        'remaining_pieces': len(game['pieces'])
    })
    
    # Simular algumas jogadas
    max_turns = 20
    turn = 0
    
    while turn < max_turns and game['status'] != 'finished':
        turn += 1
        
        # Se for a vez do jogador de teste
        if game['current_player'] == test_username:
            # Encontrar uma peça jogável
            player_pieces = game['player_pieces'][test_username]
            playable_piece = None
            position = None
            
            if not game['board']:
                # Primeira jogada - qualquer peça
                if player_pieces:
                    playable_piece = player_pieces[0]
                    position = 'right'
            else:
                # Procurar uma peça que combine com as extremidades do tabuleiro
                for piece in player_pieces:
                    if piece[0] == game['board_ends'][0]:
                        playable_piece = piece
                        position = 'left'
                        break
                    elif piece[1] == game['board_ends'][0]:
                        playable_piece = piece
                        position = 'left'
                        break
                    elif piece[0] == game['board_ends'][1]:
                        playable_piece = piece
                        position = 'right'
                        break
                    elif piece[1] == game['board_ends'][1]:
                        playable_piece = piece
                        position = 'right'
                        break
            
            if playable_piece and position:
                # Jogar a peça
                player_pieces.remove(playable_piece)
                
                if not game['board']:
                    # Primeira peça no tabuleiro
                    game['board'].append(playable_piece)
                    game['board_ends'] = [playable_piece[0], playable_piece[1]]
                else:
                    if position == 'left':
                        if playable_piece[0] == game['board_ends'][0]:
                            game['board'].insert(0, [playable_piece[1], playable_piece[0]])
                            game['board_ends'][0] = playable_piece[1]
                        else:
                            game['board'].insert(0, [playable_piece[0], playable_piece[1]])
                            game['board_ends'][0] = playable_piece[0]
                    else:  # right
                        if playable_piece[0] == game['board_ends'][1]:
                            game['board'].append([playable_piece[0], playable_piece[1]])
                            game['board_ends'][1] = playable_piece[1]
                        else:
                            game['board'].append([playable_piece[1], playable_piece[0]])
                            game['board_ends'][1] = playable_piece[0]
                
                # Registrar a jogada
                game['test_log'].append({
                    'action': 'player_move',
                    'turn': turn,
                    'piece': playable_piece,
                    'position': position,
                    'board': list(game['board']),
                    'board_ends': list(game['board_ends']),
                    'remaining_pieces': len(player_pieces)
                })
                
                # Verificar se o jogador ganhou
                if not player_pieces:
                    game['status'] = 'finished'
                    game['winner'] = test_username
                    game['test_log'].append({
                        'action': 'game_over',
                        'winner': test_username,
                        'reason': 'player_out_of_pieces'
                    })
                    break
                
                # Passar a vez para o bot
                game['current_player'] = 'Bot'
            else:
                # Não pode jogar, tentar comprar
                if game['pieces']:
                    new_piece = game['pieces'].pop()
                    player_pieces.append(new_piece)
                    
                    game['test_log'].append({
                        'action': 'player_draw',
                        'turn': turn,
                        'piece': new_piece,
                        'remaining_pieces': len(game['pieces'])
                    })
                    
                    # Verificar se pode jogar com a nova peça
                    can_play = False
                    if game['board']:
                        can_play = (new_piece[0] == game['board_ends'][0] or 
                                   new_piece[0] == game['board_ends'][1] or 
                                   new_piece[1] == game['board_ends'][0] or 
                                   new_piece[1] == game['board_ends'][1])
                    else:
                        can_play = True
                    
                    if not can_play:
                        # Passar a vez para o bot
                        game['current_player'] = 'Bot'
                else:
                    # Não há mais peças para comprar, passar a vez
                    game['current_player'] = 'Bot'
                    game['test_log'].append({
                        'action': 'player_pass',
                        'turn': turn,
                        'reason': 'no_playable_pieces'
                    })
        
        # Se for a vez do bot
        elif game['current_player'] == 'Bot':
            # Fazer a jogada do bot
            bot_played = bot_play(game)
            
            # Registrar a jogada do bot
            if game['bot_just_played']:
                game['test_log'].append({
                    'action': 'bot_move',
                    'turn': turn,
                    'piece': game['bot_move'],
                    'board': list(game['board']),
                    'board_ends': list(game['board_ends']),
                    'remaining_pieces': len(game['player_pieces']['Bot'])
                })
            else:
                game['test_log'].append({
                    'action': 'bot_pass',
                    'turn': turn,
                    'reason': 'no_playable_pieces'
                })
            
            # Verificar se o bot ganhou
            if not game['player_pieces']['Bot']:
                game['status'] = 'finished'
                game['winner'] = 'Bot'
                game['test_log'].append({
                    'action': 'game_over',
                    'winner': 'Bot',
                    'reason': 'bot_out_of_pieces'
                })
                break
            
            # Passar a vez para o jogador
            game['current_player'] = test_username
    
    # Se o jogo não terminou após o número máximo de turnos
    if game['status'] != 'finished':
        game['status'] = 'finished'
        
        # Determinar o vencedor pelo número de peças
        player_points = sum(sum(piece) for piece in game['player_pieces'][test_username])
        bot_points = sum(sum(piece) for piece in game['player_pieces']['Bot'])
        
        if player_points < bot_points:
            game['winner'] = test_username
        else:
            game['winner'] = 'Bot'
        
        game['test_log'].append({
            'action': 'game_over',
            'winner': game['winner'],
            'reason': 'max_turns_reached',
            'player_points': player_points,
            'bot_points': bot_points
        })
    
    # Retornar os resultados do teste
    return jsonify({
        'success': True,
        'game_id': game_id,
        'test_username': test_username,
        'status': game['status'],
        'winner': game['winner'],
        'board': game['board'],
        'player_pieces': game['player_pieces'][test_username],
        'bot_pieces': game['player_pieces']['Bot'],
        'log': game['test_log']
    })

@app.route('/test')
def test_page():
    return render_template('test_bot.html')

# Limpar jogos antigos periodicamente
def cleanup_old_games():
    current_time = time.time()
    to_remove = []
    
    for game_id, game in games.items():
        # Remover jogos com mais de 1 hora
        if current_time - game.get('created_at', 0) > 3600:
            to_remove.append(game_id)
    
    for game_id in to_remove:
        logger.info(f"Removendo jogo antigo: {game_id}")
        del games[game_id]

# Iniciar o servidor
if __name__ == '__main__':
    # Criar pasta para arquivos estáticos se não existir
    os.makedirs('static/sounds', exist_ok=True)
    
    # Verificar se os arquivos de som existem, caso contrário criar vazios
    sound_files = ['select.mp3', 'place.mp3', 'error.mp3', 'win.mp3']
    for sound in sound_files:
        sound_path = os.path.join('static/sounds', sound)
        if not os.path.exists(sound_path):
            with open(sound_path, 'wb') as f:
                pass  # Criar arquivo vazio
    
    # Agendar limpeza de jogos antigos a cada 10 minutos
    import threading
    def cleanup_scheduler():
        while True:
            time.sleep(600)  # 10 minutos
            cleanup_old_games()
    
    cleanup_thread = threading.Thread(target=cleanup_scheduler)
    cleanup_thread.daemon = True
    cleanup_thread.start()
    
    logger.info("Servidor iniciado")
    app.run(debug=True) 